import os
import numpy as np

def int_reader(folder):
    return int(open(os.path.join(folder, 'value')).read())

def float_reader(folder):
    return float(open(os.path.join(folder, 'value')).read())

def two_d_matrix_reader(folder):
    A = np.load('A.npy')
    columns = None
    rows = None
    columns_file = os.path.join(folder, 'columns')
    rows_file = os.path.join(folder, 'rows')
    if os.path.exists(columns_file):
        columns = open(columns_file).read().strip().split('\n')
    if os.path.exists(rows_file):
        rows = open(rows_file).read().strip().split('\n')
    return {'A': A, 'rows': rows, 'columns': columns}

data_types_readers = {
    'int': int_reader,
    'float': float_reader,
    '2d-matrix': two_d_matrix_reader
}


import os
import glob
import json
import yaml
import uuid
import shutil
import hashlib
import pandas as pd
from os.path import join
import huggingface_hub as hh
from .settings import config

def set_stage(stage):
    candidates = ['simple', 'test', 'dev', 'stable', 'publish', 'axiom_test']
    '''
    simple: to start over at any time, nothing will be lost when deleted
    test: testing some functions of axiom or to see how it works, suggesting delete time: daily
    develop: developing a project
    stable: the development is stable
    publish: publishing as a jounal paper, not deleting
    axiom_test: testing axiom itself
    '''
    assert stage in candidates
    stage_file = os.path.join(config['axiom_path'], 'stage')
    open(stage_file, 'w').write(stage)

    working_folder = get_working_folder(stage)
    if os.path.isdir(working_folder):
        shutil.rmtree(working_folder)
    subs = ['obj', 'info', 'sha256_info']
    for sub in subs:
        folder = join(working_folder, sub)
        if not os.path.isdir(folder):
            os.makedirs(folder)


def get_stage():
    stage_file = os.path.join(config['axiom_path'], 'stage')
    return open(stage_file).read()


fs_func = {} # file system func for local, huggingface, and bypy
def get_server():
    server_setting_file = os.path.join(config['axiom_path'], 'server')
    return open(server_setting_file).read()

def set_server(server):
    candidates = ['local', 'huggingface', 'bypy']
    assert server in candidates
    server_setting_file = os.path.join(config['axiom_path'], 'server')
    open(server_setting_file, 'w').write(server)

def get_sha256_from_server_by_given_name(given_name):
    setting_file = os.path.join(config['home'], '.huggingface', 'setting.json')
    hh_setting = json.load(open(setting_file)) # huggingface setting
    cache_dir = config['huggingface_cache']
    repo_id_info = hh_setting['user_name']+'/'+hh_setting['info_db']
    sha = hh.dataset_info(repo_id_info).sha
    info_folder = os.path.join(config['huggingface_cache'], 'datasets--'+repo_id_info.replace('/','--'), 'snapshots', sha)
    full_info_folder = os.path.join(info_folder, given_name)
    if not os.path.isdir(full_info_folder):
        hh.snapshot_download(repo_id_info, repo_type='dataset', cache_dir=cache_dir, token=True)
    if not os.path.isdir(full_info_folder):
        return None
    info_files = os.listdir(full_info_folder)
    if len(info_files) > 1:
        print('multiple sha256 files share the same given name', info_files, given_name)
    info_file = os.path.join(full_info_folder, info_files[0])

    ya = yaml.safe_load(open(info_file))
    sha256 = ya['sha256']
    target_info_folder = os.path.join(config['axiom_path'], 'info', 'local', given_name)
    if not os.path.isdir(target_info_folder):
        os.makedirs(target_info_folder)
    target_info_file = os.path.join(target_info_folder, sha256+'.yaml')
    yaml.safe_dump(ya, open(target_info_file,'w'))
    cache_info_folder = os.path.join(config['axiom_cache'], sha256)
    if not os.path.isdir(cache_info_folder):
        os.mkdir(cache_info_folder)
    cache_info_file = os.path.join(cache_info_folder, 'info.yaml')
    yaml.safe_dump(ya, open(target_info_file,'w'))
    yaml.safe_dump(ya, open(cache_info_file,'w'))
    return ya['sha256']



def get_uuid_by_given_name_in_folder(folder, given_name):
    fs = glob.glob(os.path.join(folder, given_name, '*.yaml'))
    ys = [yaml.safe_load(open(f).read()) for f in fs]
    idx = [i for i in range(len(fs)) if ys[i]['given_name'] == given_name]
    if len(idx) >= 1:
        return os.path.split(fs[idx[0]])[-1][:-5]
    else:
        return None


def get_uuid_by_given_name(given_name):
    axiom_path = config['axiom_path']
    md5 = get_uuid_by_given_name_in_folder(os.path.join(axiom_path, 'info', 'local'), given_name)
    if md5 is not None:
        return md5
    print('not found in local path: %s, checking with huggingface' % axiom_path)
    md5 = get_sha256_from_server_by_given_name(given_name)
    if md5 is None:
        print('get_uuid_by_given_name return None')
    return md5

def get_folder_fs(folder):
    fs = []
    for subdir, _, files in os.walk(folder):
        fs += [os.path.join(subdir, f) for f in files]
    return fs

def get_folder_sha256(folder):
    md5 = hashlib.sha256()
    size_ = 0
    for subdir, _, files in os.walk(folder):
        for f in files:
            full_name = os.path.join(subdir, f)
            size_ += os.stat(full_name).st_size
            with open(full_name, 'rb') as f:
                for chunk in iter(lambda: f.read(4096), b""):
                    md5.update(chunk)
    return md5.hexdigest(), size_



def add_obj_common(folder, to_move=True): # folder with all data but not info.yaml
    # to copy if not to_move,
    # when added form __import the folder is already at axiom cache
    uuid_folder = os.path.split(folder)[0]
    info_file = os.path.join(uuid_folder, 'info.yaml')
    info_yaml = yaml.safe_load(open(info_file))
    if not os.path.exists(info_file):
        if info_file == 'info.yaml':
            info_file = os.path.join('.', info_file)
        print("make sure %s exists" % info_file)
        return None
    obj_folder = os.path.split(folder)[-1]
    cmd = 'cd '+folder+'&& tar zfc ../'+obj_folder+'.tar.gz ./'
    print(cmd)
    os.system(cmd)
    md5, folder_size = get_folder_md5(folder)
    gz_size = os.stat(folder+'.tar.gz').st_size
    info_yaml['original_size'] = folder_size
    info_yaml['compressed_size'] = gz_size
    info_yaml['sha256'] = md5
    shutil.move(folder+'.tar.gz', os.path.join(config['axiom_path'], 'obj', md5))

    if to_move:
        md5_folder = os.path.join(config['axiom_cache'], md5)
        shutil.move(uuid_folder, md5_folder)
    else:
        target_folder = os.path.join(config['axiom_cache'], md5)
        if not os.path.isdir(target_folder):
            shutil.copytree(uuid_folder, target_folder)
    given_name = info_yaml['given_name']
    target_info_path = os.path.join(config['axiom_path'], 'info', 'local', given_name)
    if not os.path.isdir(target_info_path):
        os.makedirs(target_info_path)
    f = os.path.join(target_info_path, md5+'.yaml')
    yaml.safe_dump(info_yaml, open(f,'w'))
    upload_to_server(md5, info_yaml['given_name'])
    print(md5, 'added')

## todo, what if multiple sha256s share same given name or multiple given names share same sha256

def put_node_local(compressed_obj_file, info_yaml, sha256, given_name): # node means obj + info
    stage = get_stage()
    working_folder = get_working_folder(stage)
    shutil.move(compressed_obj_file, os.path.join(working_folder, 'obj', sha256))
    target_info_file = os.path.join(working_folder, 'info', given_name+'.yaml')
    target_info_folder = os.path.split(target_info_file)[0]
    if not os.path.isdir(target_info_folder):
        os.makedirs(target_info_folder)
    target_sha256_info_file = os.path.join(working_folder, 'sha256_info', sha256+'.yaml')
    yaml.safe_dump(info_yaml, open(target_info_file, 'w'))
    yaml.safe_dump(info_yaml, open(target_sha256_info_file, 'w'))



def add_obj_server(folder): # folder with all data but not info.yaml
    server = get_server()
    uuid_folder = os.path.split(folder)[0]
    info_file = os.path.join(uuid_folder, 'info.yaml')
    info_yaml = yaml.safe_load(open(info_file))
    if not os.path.exists(info_file):
        if info_file == 'info.yaml':
            info_file = os.path.join('.', info_file)
        print("make sure %s exists" % info_file)
        return None
    obj_folder = os.path.split(folder)[-1]
    cmd = 'cd '+folder+'&& tar zfc ../'+obj_folder+'.tar.gz ./'
    print(cmd)
    os.system(cmd)
    sha256, folder_size = get_folder_sha256(folder)
    gz_size = os.stat(folder+'.tar.gz').st_size
    info_yaml['original_size'] = folder_size
    info_yaml['compressed_size'] = gz_size
    info_yaml['sha256'] = sha256
    compressed_obj_file = folder+'.tar.gz'
    given_name = info_yaml['given_name']
    yaml.safe_dump(info_yaml, open(info_file, 'w'))
    fs_func[server]['put_node'](compressed_obj_file, info_yaml, sha256, given_name)
    if os.path.exists(compressed_obj_file):# when server is local, just move, this might not exist
        os.remove(compressed_obj_file)
    print(sha256, 'added')


def get_sha256_by_given_name_local(given_name):
    working_folder = get_working_folder(get_stage())
    info_folder = os.path.join(working_folder, 'info', given_name+'.yaml')
    return yaml.safe_load(open(info_folder))['sha256']

def download_node_from_server_local(sha256, given_name):
    extract_folder = os.path.join(config['axiom_cache'], sha256, 'obj')
    working_folder = get_working_folder(get_stage())
    if not os.path.isdir(extract_folder):
        os.makedirs(extract_folder)
        sha_full_path = os.path.join(working_folder, 'obj', sha256)
        cmd = 'tar zxf %s -C %s' % (sha_full_path, extract_folder)
        print(cmd)
        os.system(cmd)
    source_info_file = os.path.join(working_folder, 'info', given_name+'.yaml')
    target_info_file = os.path.join(config['axiom_cache'], sha256, 'info.yaml')
    shutil.copy(source_info_file, target_info_file)
    return extract_folder

def list_axiom(fs):
    ys = [yaml.safe_load(open(f).read()) for f in fs]
    sha256s = [y['sha256'][:10]+'...' for y in ys]
    columns = ['given_name', 'type', 'size', 'description']
    d = {}
    for i in range(len(fs)):
        d[sha256s[i]] = [ys[i]['given_name'], ys[i]['type'], ys[i]['compressed_size'], ys[i]['description'][:10]]
    df = pd.DataFrame.from_dict(d, orient='index', columns=columns)
    print(df)


def axiom_ls(base_folder, path, ignores=[]):
    if path is None:
        cwd = open(get_cwd_file()).read()
        folder = join(base_folder, cwd)
        folders = os.listdir(folder)
        folders = [folder for folder in folders if not folder in ignores]
        print('\n'.join(folders))
    else:
        folder = os.path.join(base_folder, path)
        fs = get_folder_fs(folder)
        list_axiom(fs)

def ls_local(path, ignores=[]):
    stage = get_stage()
    base_folder = join(config['axiom_path'], 'stages', stage)
    axiom_ls(base_folder, path, ignores)

def get_sha256_by_given_name_huggingface(given_name):
    cache_dir = config['huggingface_cache']
    repo_id = get_repo_id('info')
    hh.hf_hub_download(repo_id=repo_id, filename=given_name+'.yaml', repo_type="dataset", cache_dir=cache_dir, token=True)
    sha = hh.dataset_info(repo_id).sha
    info_file = os.path.join(config['huggingface_cache'], 'datasets--'+repo_id.replace('/','--'), 'snapshots', sha, given_name+'.yaml')
    return yaml.safe_load(open(info_file))['sha256']

fs_func['local'] = {
    'get_sha256_by_given_name': get_sha256_by_given_name_local,
    'put_node': put_node_local,
    'download_node_from_server': download_node_from_server_local,
    'ls': ls_local
}

def put_node_huggingface(compressed_obj_file, info_yaml, sha256, given_name): # node means obj + info
    repo_id = get_repo_id('obj')
    hh.upload_file(path_or_fileobj=compressed_obj_file,path_in_repo=sha256,repo_id=repo_id,repo_type="dataset",token=True)
    tmp_file = str(uuid.uuid4())
    yaml.safe_dump(info_yaml,open(tmp_file, 'w'))
    hh.upload_file(path_or_fileobj=tmp_file,path_in_repo=sha256+'.yaml',repo_id=get_repo_id('sha256_info'),repo_type="dataset",token=True)
    hh.upload_file(path_or_fileobj=tmp_file,path_in_repo=given_name+'.yaml',repo_id=get_repo_id('info'),repo_type="dataset",token=True)
    os.remove(tmp_file)

def download_node_from_server_huggingface(sha256, given_name):
    setting_file = os.path.join(config['home'], '.huggingface', 'setting.json')
    hh_setting = json.load(open(setting_file)) # huggingface setting
    cache_dir = config['huggingface_cache']
    repo_id = hh_setting['user_name']+'/'+hh_setting['obj_db']
    hh.hf_hub_download(repo_id=repo_id, filename=sha256, repo_type="dataset", cache_dir=cache_dir, token=True)

    sha = hh.dataset_info(repo_id).sha
    obj_folder = os.path.join(config['huggingface_cache'], 'datasets--'+repo_id.replace('/','--'), 'snapshots', sha)
    extract_folder = os.path.join(config['axiom_cache'], sha256, 'obj')
    if not os.path.isdir(extract_folder):
        os.makedirs(extract_folder)
        md5_full_path = os.path.join(obj_folder, sha256)
        cmd = 'tar zxf %s -C %s' % (md5_full_path, extract_folder)
        print(cmd)
        os.system(cmd)
    repo_id = hh_setting['user_name']+'/'+hh_setting['info_db']
    hh.hf_hub_download(repo_id=repo_id, filename=given_name+'.yaml', repo_type="dataset", cache_dir=cache_dir, token=True)
    sha = hh.dataset_info(repo_id).sha
    source_info_file = os.path.join(config['huggingface_cache'], 'datasets--'+repo_id.replace('/','--'), 'snapshots', sha, given_name+'.yaml')
    target_info_file = os.path.join(config['axiom_cache'], sha256, 'info.yaml')
    shutil.copy(source_info_file, target_info_file)
    return extract_folder


def get_info_huggingface(update=False): # return the info folder
    stage = get_stage()
    user_name = open(os.path.join(config['home'], '.huggingface', 'user_name')).read()
    cache_dir = config['huggingface_cache']
    repo_id_info = user_name+'/'+stage+'_info'
    sha_file = os.path.join(config['home'], '.huggingface', stage+'_info.sha')
    if update:
        hh.snapshot_download(repo_id_info, repo_type='dataset', cache_dir=cache_dir, token=True)
        sha = hh.dataset_info(repo_id_info).sha
        open(sha_file, 'w').write(sha)
    else:
        if os.path.exists(sha_file):
            sha = open(sha_file).read()
        else:
            sha = hh.dataset_info(repo_id_info).sha
            open(sha_file, 'w').write(sha)
    info_folder = os.path.join(config['huggingface_cache'], 'datasets--'+repo_id_info.replace('/','--'), 'snapshots', sha)
    return info_folder

def ls_huggingface(path, update=False):
    info_folder = get_info_huggingface(update)
    print('info_folder', info_folder)
    axiom_ls(info_folder, path, ignores=['.gitattributes'])

def rm_huggingface(given_name):
    sha256 = get_sha256_by_given_name_huggingface(given_name)
    setting_file = os.path.join(config['home'], '.huggingface', 'setting.json')
    hh_setting = json.load(open(setting_file)) # huggingface setting
    repo_id = hh_setting['user_name']+'/'+hh_setting['info_db']
    hh.delete_file(given_name+'.yaml', repo_id=repo_id, repo_type='dataset', token=True)
    repo_id = hh_setting['user_name']+'/'+hh_setting['obj_db']
    hh.delete_file(sha256, repo_id=repo_id, repo_type='dataset', token=True)
    repo_id = hh_setting['user_name']+'/'+hh_setting['sha256_info_db']
    hh.delete_file(sha256+'.yaml', repo_id=repo_id, repo_type='dataset', token=True)

fs_func['huggingface'] = {
    'get_sha256_by_given_name': get_sha256_by_given_name_huggingface,
    'put_node': put_node_huggingface,
    'download_node_from_server': download_node_from_server_huggingface,
    'ls': ls_huggingface,
    'rm': rm_huggingface,
}

def download_node_from_server(sha256, given_name):
    server = get_server()
    fs_func[server]['download_node_from_server'](sha256, given_name)

def get_sha256_by_given_name(given_name):
    server = get_server()
    return fs_func[server]['get_sha256_by_given_name'](given_name)

def get_folder_by_sha256(sha256, server='local'):
    cache_folder = os.path.join(config['axiom_cache'], sha256, 'obj')
    if os.path.isdir(cache_folder):
        return cache_folder
    else:
        download_node_from_server(sha256, server) # download obj and info from server
        if os.path.isdir(cache_folder):
            return cache_folder
    return None


def get_folder_by_given_name(given_name):
    sha256s = glob.glob(os.path.join(config['axiom_cache'], '*'))
    yas = [yaml.safe_load(open(os.path.join(sha256, 'info.yaml'))) for sha256 in sha256s]
    yas = [y for y in yas if y['given_name'] == given_name]
    if len(yas) == 0:
        sha256 = get_sha256_by_given_name(given_name)
        if sha256 is None:
            return None
        download_node_from_server(sha256, given_name)
    else:
        if len(yas) > 1:
            print('warning, multiple sha256s share with the same given_name', given_name)
            print('the are')
            print('\n'.join([y['sha256'] for y in yas]))
            print('only the first one is selected')
        sha256 = yas[0]['sha256']
    cache_folder = os.path.join(config['axiom_cache'], sha256, 'obj')
    return cache_folder

def rm(given_name):
    server = get_server()
    fs_func[server]['rm'](given_name)

def get_working_folder(stage): # for local only
    return join(config['axiom_path'], 'stages', stage)

def get_cwd_file():
    stage = get_stage()
    return join(config['axiom_path'], 'stages', stage, 'cwd')


def reset_local():
    stage = get_stage()
    assert stage in ['simple', 'test']
    working_folder = get_working_folder(stage)
    if os.path.isdir(working_folder):
        shutil.rmtree(working_folder)
    subs = ['obj', 'info', 'sha256_info']
    for sub in subs:
        folder = join(working_folder, sub)
        print('creating folder', folder)
        os.makedirs(folder)


def get_repo_id(what): # what in ['info', 'obj', 'sha256_info']
    stage = get_stage()
    user_name = open(os.path.join(config['home'], '.huggingface', 'user_name')).read()
    repo_id = user_name+'/'+stage+'_'+what
    return repo_id

def reset_one(what):
    stage = get_stage()
    assert stage in ['simple', 'test']
    repo_id = get_repo_id(what)
    try:
        hh.delete_repo(repo_id=repo_id, repo_type='dataset', token=True)
    except Exception as e:
        print(e)
    hh.create_repo(repo_id, repo_type='dataset', token=True)


def reset_huggingface():
    stage = get_stage()
    assert stage in ['simple', 'test']
    whats = ['info', 'obj', 'sha256_info']
    for what in whats:
        reset_one(what)


def reset():
    server = get_server()
    if server == 'local':
        reset_local()
    elif server == 'huggingface':
        reset_huggingface()

def getcwd():
    return open(get_cwd_file()).read()

def ls(sub=None):
    server = get_server()
    fs_func[server]['ls'](sub)

def show(given_name):
    folder = get_folder_by_given_name(given_name)
    sha256_folder = os.path.split(folder)[0]
    info_file = join(sha256_folder, 'info.yaml')
    print(open(info_file).read())

def set_cwd(folder):
    open(get_cwd_file(), 'w').write(folder)


import os
import json
from .settings import config

for k in config:
    if k.endswith('_path') or k.endswith('_cache'):
        if not os.path.isdir(config[k]):
            os.makedirs(config[k])

f = os.path.join(config['home'], '.huggingface/token')
if os.path.exists(f):
    token = open(f).read()
else:
    token = 'please run python -m axiom huggingface add token to '+f

server_file = os.path.join(config['axiom_path'], 'server')
if not os.path.exists(server_file):
    open(server_file, 'w').write('local')

stage_file = os.path.join(config['axiom_path'], 'stage')
if not os.path.exists(stage_file):
    open(stage_file, 'w').write('simple')

stages = ['simple', 'test', 'dev', 'stable']
for stage in stages:
    folder = os.path.join(config['axiom_path'], 'stages', stage)
    if not os.path.isdir(folder):
        os.makedirs(folder)
    cwd_file = os.path.join(folder, 'cwd')
    if not os.path.exists(cwd_file):
        open(cwd_file, 'w').write('')

setting_file = os.path.join(config['axiom_path'], 'settings.json')
if not os.path.exists(server_file):
    prefix = ['simple', 'test', 'dev', 'stable']
    d = {}
    for p in prefix:
        d[p] = os.path.join(config['axiom_path'], 'stages', p)
    json.dump(d, open(setting_file, 'w'))





from axiom.__data import main as __data
from axiom.__import import main as __import

int_3 = __data('axiom/example/int_3')
f = __import('axiom/example/int_add_one_and_two')

r1, r2 = f({}, int_3, given_name=['axiom/example/int_4', 'axiom/example/int_5'])
print('result', r1, r2)





from axiom.__data import main as __data
from axiom.__import import main as __import

int_3 = __data('axiom/example/int_3')
add_one = __import('axiom/example/int_add_one')

result = add_one({}, int_3, given_names='axiom/example/int_4')

print('result', result)



import os
__home = os.environ['HOME']
config = {
    'home': __home,
    'axiom_path': os.path.join(__home, 'reaxiom'),
    'axiom_cache': os.path.join(__home, 'reaxiom', 'cache'),
    'huggingface_cache': os.path.join(__home, 'reaxiom', 'huggingface_cache'),
    'bypy_cache': os.path.join(__home, 'reaxiom', 'bypy_cache')
}



##__AXIOM_BLOCK_BEGIN__
block_info = {
    "type": "function",
    "given_name": "axiom/example/int_add",
    "input": ["data.int","data.int"],
    "output": "data.int",
    "axiom_name": "__AXIOM__",
    "description": "add two integers"
}


##__AXIOM_INFO_END__
import os

def int_reader(folder):
    return int(open(os.path.join(folder, 'value')).read())

def int_writer(folder, value):
    assert isinstance(value, int)
    open(os.path.join(folder, 'value'), 'w').write(str(value))

def main(config, in_folder, given_name, out_folder):
    assert len(in_folder) == 2
    value1 = int_reader(in_folder[0])
    value2 = int_reader(in_folder[1])
    int_writer(out_folder, value1+value2)
    return out_folder

##__AXIOM_FUNCTION_END__


if __name__ == '__main__':
    out_folder = '/tmp/tmp/a'
    data1 = '/Users/test/reaxiom/cache/eccbc87e4b5ce2fe28308fd9f2a7baf3/obj'
    data2 = '/Users/test/reaxiom/cache/e4da3b7fbbce2345d7772b0674a318d5/obj'
    
    output = main(
        config={},
        in_folder=[data1, data2],
        given_name = 'axiom/example/int_some_number_1',
        out_folder = out_folder
    )
    print(output)
##__AXIOM_BLOCK_END__


import os
from .settings import config
from .common import get_folder_by_given_name, get_folder_by_sha256
def main(db, by='given_name'):
    if by == 'given_name':
        folder = get_folder_by_given_name(db) # in which get cache first, then download from server, local is one type os server
    else:
        folder = get_folder_by_sha256(db)
    return folder



from axiom.__data import main as __data
from axiom.__import import main as __import

int_3 = __data('axiom/example/int_3')
int_5 = __data('axiom/example/int_5')
f = __import('axiom/example/int_add')

result = f({}, [int_3, int_5], given_name='axiom/example/some_number_1')
print('result', result)





from axiom.__data import main as __data
from axiom.__import import main as __import

int_3 = __data('axiom/example/int_3')
int_4 = __data('axiom/example/int_4')
add = __import('axiom/example/int_add')

add_one = __import('axiom/example/int_add_one')
add_one_and_two = __import('axiom/example/int_add_one_and_two')
add_by_config = __import('axiom/example/int_add_by_config')
result = add({}, [int_3, int_4], given_name='axiom/example/some_number_1')
r2 = add({}, [int_3, result], given_name='axiom/example/some_number_2')
r3 = add_one({}, r2, given_name='axiom/example/some_number_3')
r4, r5 = add_one_and_two({}, r3, given_name=['axiom/example/a', 'axiom/example/b'])
r6 = add_by_config({'n':2}, r5, given_name='axiom/example/c')




import os
import sys
import uuid
import yaml
import json
import glob
import shutil
import getpass
import pandas as pd
import axiom.common
from itertools import chain
from axiom.settings import config



def main(sys_argv):
    shell_pid = str(os.getppid())
    global_value_file = os.path.join(config['axiom_path'], '.pid'+shell_pid)
    if not os.path.exists(global_value_file):
        cwd = ''
        global_value = {'cwd': cwd}
        json.dump(global_value, open(global_value_file,'w'))
    if len(sys_argv) == 1:
        lines = open(__file__).read().split('\n')
        lines = [line for line in lines if line.find('if sys_argv[1]') > 0][1:]
        cmds = [line.split("'")[1].split("'")[0] for line in lines]
        comments = [line.split("#")[-1] for line in lines]
        usages = [cmds[i]+': '+comments[i] for i in range(len(cmds))]
        print('\n'.join(usages))
        return

    if sys_argv[1] == 'info': # show basic info of the project
        print('my file name', __file__)
        print('config', config)

    def compile(txt):
        from axiom.settings import config
        folder = str(uuid.uuid4())
        os.makedirs(os.path.join(folder, 'obj'))
        me, code_txt = txt.split('##__AXIOM_INFO_END__\n')
        code_txt = code_txt.split('##__AXIOM_FUNCTION_END__')[0]
        print('me'.center(50,'='))
        print(me)
        print('code'.center(50,'='))
        print(code_txt)
        open(os.path.join(folder, 'obj', 'main.py'),'w').write(code_txt)
        info_file = os.path.join(folder, 'info.yaml')

        me = me[me.find('=')+1:]
        yaml.safe_dump(json.loads(me), open(info_file, 'w'))
        print('info_file', info_file)
        axiom.common.add_obj_server(os.path.join(folder, 'obj'))
        shutil.rmtree(folder)

    if sys_argv[1] == 'compile':
        txt = open(sys_argv[2]).read()
        blocks = txt.split('##__AXIOM_BLOCK_BEGIN__')[1:]
        blocks = [b.split('##__AXIOM_BLOCK_END__')[0] for b in blocks]
        for b in blocks:
            compile(b)

    if sys_argv[1] == 'what': # show what can we show
        fs = glob.glob(os.path.join(config['axiom_path'], 'info', '*'))
        yamls = [yaml.safe_load(open(f).read()) for f in fs]
        if len(sys_argv) == 2:
            keys = [list(y.keys()) for y in yamls]
            print(list(set(list(chain.from_iterable(keys)))))
        if len(sys_argv) == 3:
            obj_properties = []
            for y in yamls:
                x = y[sys_argv[2]]
                if isinstance(x, list):
                    obj_properties += x
                else:
                    obj_properties.append(x)

            obj_properties = sorted(list(set(obj_properties)))
            print('\n'.join(obj_properties))

    if sys_argv[1] == 'template': # axiom template report_matrix, add a function template at the current folder
        func_name = sys_argv[2]
        os.mkdir(func_name)
        open(os.path.join(func_name, '__main.py'), 'w').write(template_main)
        open(os.path.join(func_name, 'info.yaml'), 'w').write(template_info)
        open(os.path.join(func_name, '__INCLUDE__'), 'w').write('external folder')
        open(os.path.join(func_name, 'config.yaml'), 'w').write('{}')

    if sys_argv[1] == 'ls_old': # show the axiom.obj
        if len(sys_argv) == 3:
            axiom_path = sys_argv[2]
        else:
            axiom_path = config['axiom_path']
        list_axiom(os.path.join(axiom_path, 'info'))

    if sys_argv[1] == 'lsb': # list all the objects at huggingface axiom/info
        list_axiom(axiom.common.config['bypy_cache'])

    if sys_argv[1] == 'add': # add the current folder as the obj
        axiom.common.add_obj_server(sys_argv[2])

    if sys_argv[1] == 'rm': # remove an axiom object by given name
        axiom.common.rm(sys_argv[2])

    if sys_argv[1] == 'extract': # exting an object to the corrent folder
        md5 = sys_argv[2]
        f = os.path.join(config['axiom_path'], 'obj', md5)
        cmd = 'cp %s ./&& tar zfx %s&&rm %s' % (f, md5, md5)
        print(cmd)
        os.system(cmd)


    if sys_argv[1] == 'commit': # upload an md5 to bypy
        md5 = sys_argv[2]
        axiom.common.upload_to_server(md5)

    def run_python_block(txt):
        file_name = str(uuid.uuid4())
        open(file_name, 'w').write(txt)
        os.system('python '+file_name)
        os.remove(file_name)

    if sys_argv[1] == 't': # test a file with __AXIOM__BLOCK__BEGIN__ block, with test
        txt = open(sys_argv[2]).read()
        blocks = txt.split('##__AXIOM_BLOCK_BEGIN__')[1:]
        blocks = [b.split('##__AXIOM_BLOCK_END__')[0] for b in blocks]
        for b in blocks:
            run_python_block(b)

    if sys_argv[1] == 'pwd':
        print(axiom.common.getcwd())

    if sys_argv[1] == 'ls':
        if len(sys_argv) == 2:
            axiom.common.ls()
        else:
            axiom.common.ls(sys_argv[2])

    if sys_argv[1] == 'ls.update':
        if len(sys_argv) == 2:
            axiom.common.fs_func[axiom.common.get_server()]['ls'](None, update=True)
        else:
            axiom.common.fs_func[axiom.common.get_server()]['ls'](sys_argv[2], update=True)

    if sys_argv[1] == 'cd':
        if len(sys_argv) == 2:
            target_folder = ""
            axiom.common.set_cwd(target_folder)
        else:
            working_folder = axiom.common.get_working_folder(axiom.common.get_stage())
            sub = sys_argv[2]
            cwd = open(axiom.common.get_cwd_file()).read()
            if sub == '..':
                target_folder = os.path.split(cwd)[0]
            else:
                target_folder = os.path.join(cwd, sub)
            if os.path.isdir(os.path.join(working_folder, 'info', target_folder)):
                axiom.common.set_cwd(target_folder)
            else:
                print('not found:', target_folder)

    if sys_argv[1] == 'server': # set the server, from [local, huggingface, bypy] or get the server
        if len(sys_argv) == 2:
            print(axiom.common.get_server())
        else:
            axiom.common.set_server(sys_argv[2])


    if sys_argv[1] == 'huggingface': # settup the huggingface token and username
        token = getpass.getpass('huggingface token:')
        user_name = input('huggingface private db user:')
        huggingface_path = os.path.join(config['home'], '.huggingface')
        if not os.path.isdir(huggingface_path):
            os.mkdir(huggingface_path)
        open(os.path.join(huggingface_path, 'token'), 'w').write(token)
        open(os.path.join(huggingface_path, 'user_name'), 'w').write(user_name)


    if sys_argv[1] == 'data.info': ## add info.yaml template to the current folder
        s = '''axiom_name: __AXIOM__
description:
given_name: com/some-company/some-project/some-dataset
type: data.int'''
        open('info.yaml', 'w').write(s)

    if sys_argv[1] == 'stage': # ['simple', 'test', 'dev', 'stable', 'publish', 'axiom_test']
        if len(sys_argv) == 2:
            print(axiom.common.get_stage())
        else:
            axiom.common.set_stage(sys_argv[2])
    if sys_argv[1] == 'reset':
        axiom.common.reset()

    if sys_argv[1] == 'example':
        '''
            a add ./int_3/obj
            a add ./int_4/obj
            a compile add_func.py
            a compile add_one_func.py
            a compile add_one_and_two.py
        '''
        folder_me = os.path.split(__file__)[0]
        main(['a', 'add', os.path.join(folder_me, 'example', 'simple_add', 'int_3', 'obj')])
        main(['a', 'add', os.path.join(folder_me, 'example', 'simple_add', 'int_4', 'obj')])
        main(['a', 'compile', os.path.join(folder_me, 'example', 'simple_add', 'add_func.py')])
        main(['a', 'compile', os.path.join(folder_me, 'example', 'simple_add', 'add_one_func.py')])
        main(['a', 'compile', os.path.join(folder_me, 'example', 'simple_add', 'add_one_and_two.py')])
        main(['a', 'compile', os.path.join(folder_me, 'example', 'simple_add', 'add_by_config.py')])
        just_test_file = os.path.join(folder_me, 'example', 'simple_add', 'just_test.py')
        os.system('python %s' % just_test_file)
    if sys_argv[1] == 'show':
        axiom.common.show(sys_argv[2])

if __name__ == '__main__':
    #main(['a', 'rm', 'com/weizhu/ricequant/hkoriginal_10stockx2month'])
    main(sys.argv)
    #folder_me = '/Users/test/axiom/master/code/site-package/axiom'
    #main(['a', 'add', os.path.join(folder_me, 'example', 'simple_add', 'int_3', 'obj')])
